package com.utopiaindustries.controller;

import com.utopiaindustries.auth.JobCardRole;
import com.utopiaindustries.model.ctp.JobCard;
import com.utopiaindustries.service.InventoryAccountService;
import com.utopiaindustries.service.JobCardService;
import com.utopiaindustries.service.LocationService;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.util.List;

@Controller
@JobCardRole
@RequestMapping( "/job-cards" )
public class JobCardController {

    private final JobCardService jobCardService;
    private final LocationService locationService;
    private final InventoryAccountService inventoryAccountService;

    public JobCardController(JobCardService jobCardService, LocationService locationService, InventoryAccountService inventoryAccountService){
        this.jobCardService = jobCardService;
        this.locationService = locationService;
        this.inventoryAccountService = inventoryAccountService;
    }

    /**
     *  get all job cards
     * */
    @GetMapping
    public String showJobCardList(  @RequestParam( value = "id", required = false ) String id,
                                    @RequestParam( value = "code", required = false ) String code,
                                    @RequestParam( value = "status", required = false ) String status,
                                    @RequestParam( value = "inventory-status" , required = false) String inventoryStatus,
                                    @RequestParam( value = "customer" ,required = false ) String customer,
                                    @RequestParam( value = "lot-number", required = false ) String lotNumber,
                                    @RequestParam( value = "purchase-order-id", required = false ) String purchaseOrderId,
                                    @RequestParam( value = "location-site-id", required = false ) String locationSiteId,
                                    @RequestParam( value = "created-start-date", required = false ) String createdStartDate,
                                    @RequestParam( value = "created-end-date", required = false ) String createdEndDate,
                                    @RequestParam( value = "limit" , required = false) Long limit,
                                    Model model ){
        List<JobCard> cards = jobCardService.getCards(  id, code, status, inventoryStatus, customer, lotNumber, purchaseOrderId, locationSiteId,createdStartDate, createdEndDate, limit );
        model.addAttribute("cards", cards );
        model.addAttribute("statuses", JobCard.Status.values() );
        model.addAttribute("invStatuses", JobCard.InventoryStatus.values() );
        model.addAttribute("locations", locationService.findAll() );
        return "job-card-list";
    }

    @GetMapping( "/new" )
    public String showJobCardForm( Model model ){
        model.addAttribute("jobCard", jobCardService.createNewJobCard() );
        model.addAttribute("cutPieceTypes", jobCardService.getAllPieceTypes() );
        model.addAttribute("accounts", inventoryAccountService.getAllCuttingAccounts( ) );
        return "job-card-add";
    }

    @GetMapping( value = "/edit/{id}" )
    public String showJobCardEditForm( @PathVariable("id") long id,
                                       Model model ){
        model.addAttribute("jobCard", jobCardService.findCardRecursivelyForView( id ) );
        model.addAttribute("cutPieceTypes", jobCardService.getAllPieceTypes() );
        model.addAttribute("accounts", inventoryAccountService.getAllCuttingAccounts() );
        return "job-card-edit";
    }

    /*
    *  draft
    * */
    @PostMapping( value ="/edit" , params = "user=draft" )
    public String saveJobCard( @ModelAttribute JobCard jobCard,
                               RedirectAttributes redirectAttributes,
                               Model model ){

        try {
            jobCard.setStatus( JobCard.Status.DRAFT.name() );
            jobCard.setInventoryStatus( JobCard.InventoryStatus.NOT_RECEIVED_YET.name() );
            jobCardService.save( jobCard );
           redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }

    /*
    *  draft after save
    * */
    @PostMapping( value ="/edit/{id}" , params = "user=draft" )
    public String saveJobCardEdit( @ModelAttribute JobCard jobCard,
                                   @PathVariable long id,
                                   RedirectAttributes redirectAttributes,
                                   Model model ){

        try {
            jobCard.setStatus( JobCard.Status.DRAFT.name() );
            jobCard.setInventoryStatus( JobCard.InventoryStatus.NOT_RECEIVED_YET.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }

    @PostMapping( value ="/edit" , params = "user=post" )
    public String postJobCard( @ModelAttribute JobCard jobCard,
                               RedirectAttributes redirectAttributes,
                               Model model ){

        try {
            jobCard.setStatus(JobCard.Status.POSTED.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }


    /*
    * post after draft
    * */
    @PostMapping( value ="/edit/{id}" , params = "user=post" )
    public String postJobCardEdit( @ModelAttribute JobCard jobCard,
                                   @PathVariable long id,
                                   RedirectAttributes redirectAttributes,
                                   Model model ){

        try {
            jobCard.setStatus(JobCard.Status.POSTED.name() );
            jobCardService.save( jobCard );
            redirectAttributes.addFlashAttribute("success", "Successfully saved!" );
        } catch ( Exception ex ){
            redirectAttributes.addFlashAttribute("error", ex.getMessage() );
        }
        return "redirect:/job-cards";
    }
}
